@@ -17,12 +17,13 @@
 
 import asyncio
 import os
+import re
 import ssl
 import warnings
 
 import urllib3  # type: ignore
 
-from ..compat import urlencode
+from ..compat import reraise_exceptions, urlencode
 from ..connection.base import Connection
 from ..exceptions import (
     ConnectionError,
@@ -49,6 +50,15 @@ try:
 except ImportError:
     pass
 
+_version_parts = []
+for _version_part in aiohttp.__version__.split(".")[:3]:
+    try:
+        _version_parts.append(int(re.search(r"^([0-9]+)", _version_part).group(1)))  # type: ignore[union-attr]
+    except (AttributeError, ValueError):
+        break
+_AIOHTTP_SEMVER_VERSION = tuple(_version_parts)
+_AIOHTTP_FIXED_HEAD_BUG = _AIOHTTP_SEMVER_VERSION >= (3, 7, 0)
+
 
 class AsyncConnection(Connection):
     """Base class for Async HTTP connection implementations"""
@@ -77,6 +87,8 @@ class AIOHttpConnection(AsyncConnection):
         self,
         host="localhost",
         port=None,
+        url_prefix="",
+        timeout=10,
         http_auth=None,
         use_ssl=False,
         verify_certs=VERIFY_CERTS_DEFAULT,
@@ -138,6 +150,8 @@ class AIOHttpConnection(AsyncConnection):
         super().__init__(
             host=host,
             port=port,
+            url_prefix=url_prefix,
+            timeout=timeout,
             use_ssl=use_ssl,
             headers=headers,
             http_compress=http_compress,
@@ -238,14 +252,16 @@ class AIOHttpConnection(AsyncConnection):
         url_path = self.url_prefix + url
         if params:
             query_string = urlencode(params)
+            url_target = "%s?%s" % (url_path, query_string)
         else:
             query_string = ""
+            url_target = url_path
 
-        # There is a bug in aiohttp that disables the re-use
+        is_head = False
+        # There is a bug in aiohttp<3.7 that disables the re-use
         # of the connection in the pool when method=HEAD.
         # See: aio-libs/aiohttp#1769
-        is_head = False
-        if method == "HEAD":
+        if method == "HEAD" and not _AIOHTTP_FIXED_HEAD_BUG:
             method = "GET"
             is_head = True
 
@@ -296,21 +312,31 @@ class AIOHttpConnection(AsyncConnection):
                 timeout=timeout,
                 fingerprint=self.ssl_assert_fingerprint,
             ) as response:
+                response_headers = {
+                    header.lower(): value for header, value in response.headers.items()
+                }
                 if is_head:  # We actually called 'GET' so throw away the data.
                     await response.release()
                     raw_data = ""
                 else:
-                    raw_data = await response.text()
+                    raw_data = await response.read()
+                    content_type = response_headers.get("content-type", "")
+
+                    # The 'application/vnd.mapbox-vector-file' type shouldn't be
+                    # decoded into text, instead should be forwarded as bytes.
+                    if content_type != "application/vnd.mapbox-vector-tile":
+                        raw_data = raw_data.decode("utf-8", "surrogatepass")
+
                 duration = self.loop.time() - start
 
         # We want to reraise a cancellation or recursion error.
-        except (asyncio.CancelledError, RecursionError):
+        except reraise_exceptions:
             raise
         except Exception as e:
             self.log_request_fail(
                 method,
                 str(url),
-                url_path,
+                url_target,
                 orig_body,
                 self.loop.time() - start,
                 exception=e,
@@ -332,7 +358,7 @@ class AIOHttpConnection(AsyncConnection):
             self.log_request_fail(
                 method,
                 str(url),
-                url_path,
+                url_target,
                 orig_body,
                 duration,
                 status_code=response.status,
@@ -341,10 +367,10 @@ class AIOHttpConnection(AsyncConnection):
             self._raise_error(response.status, raw_data)
 
         self.log_request_success(
-            method, str(url), url_path, orig_body, response.status, raw_data, duration
+            method, str(url), url_target, orig_body, response.status, raw_data, duration
         )
 
-        return response.status, response.headers, raw_data
+        return response.status, response_headers, raw_data
 
     async def close(self):
         """
@@ -362,13 +388,16 @@ class AIOHttpConnection(AsyncConnection):
             self.loop = get_running_loop()
         self.session = aiohttp.ClientSession(
             headers=self.headers,
-            skip_auto_headers=("accept", "accept-encoding"),
+            skip_auto_headers=("accept", "accept-encoding", "user-agent"),
             auto_decompress=True,
             loop=self.loop,
             cookie_jar=aiohttp.DummyCookieJar(),
             response_class=ESClientResponse,
             connector=aiohttp.TCPConnector(
-                limit=self._limit, use_dns_cache=True, ssl=self._ssl_context
+                limit=self._limit,
+                use_dns_cache=True,
+                enable_cleanup_closed=True,
+                ssl=self._ssl_context,
             ),
         )
 
